@@ -1,20 +1,29 @@
+/*
+Copyright 2011 Clint Bellanger
+
+This file is part of FLARE.
+
+FLARE is free software: you can redistribute it and/or modify it under the terms
+of the GNU General Public License as published by the Free Software Foundation,
+either version 3 of the License, or (at your option) any later version.
+
+FLARE is distributed in the hope that it will be useful, but WITHOUT ANY
+WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
+PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+FLARE.  If not, see http://www.gnu.org/licenses/
+*/
+
 /**
  * class Avatar
  *
  * Contains logic and rendering routines for the player avatar.
- *
- * @author Clint Bellanger
- * @license GPL
  */
 #ifndef AVATAR_H
 #define AVATAR_H
 
 
-#include <sstream>
-#include "SDL.h"
-#include "SDL_image.h"
-#include "SDL_mixer.h"
-
 #include "Entity.h"
 #include "Utils.h"
 #include "InputState.h"
@@ -22,22 +31,32 @@
 #include "StatBlock.h"
 #include "Hazard.h"
 #include "PowerManager.h"
+#include "SharedResources.h"
+#include "MenuManager.h"
+
+#include <SDL.h>
+#include <SDL_image.h>
+
+
+/**
+ * Avatar State enum
+ */
+enum AvatarState {
+	AVATAR_STANCE = 0,
+	AVATAR_RUN = 1,
+	AVATAR_MELEE = 2,
+	AVATAR_BLOCK = 3,
+	AVATAR_HIT = 4,
+	AVATAR_DEAD = 5,
+	AVATAR_CAST = 6,
+	AVATAR_SHOOT = 7
+};
 
-// AVATAR State enum
-const int AVATAR_STANCE = 0;
-const int AVATAR_RUN = 1;
-const int AVATAR_MELEE = 2;
-const int AVATAR_BLOCK = 3;
-const int AVATAR_HIT = 4;
-const int AVATAR_DEAD = 5;
-const int AVATAR_CAST = 6;
-const int AVATAR_SHOOT = 7;
 
 class Avatar : public Entity {
 private:
 	
 	PowerManager *powers;
-	InputState *inp;
 
 	bool lockSwing;
 	bool lockCast;
@@ -56,12 +75,13 @@ private:
 	string img_off;
 
 public:
-	Avatar(PowerManager *_powers, InputState *_inp, MapIso *_map);
+	Avatar(PowerManager *_powers, MapIso *_map);
 	~Avatar();
 	
 	void init();
 	void loadGraphics(string img_main, string img_armor, string img_off);
 	void loadSounds();
+	void loadStepFX(string stepname);
 	
 	void logic(int actionbar_power, bool restrictPowerUse);
 	bool pressing_move();	
@@ -76,6 +96,7 @@ public:
 	int current_power;
 	Point act_target;
 	bool drag_walking;
+    bool newLevelNotification;
 };
 
 #endif
