@@ -1,10 +1,24 @@
+/*
+Copyright 2011 Clint Bellanger
+
+This file is part of FLARE.
+
+FLARE is free software: you can redistribute it and/or modify it under the terms
+of the GNU General Public License as published by the Free Software Foundation,
+either version 3 of the License, or (at your option) any later version.
+
+FLARE is distributed in the hope that it will be useful, but WITHOUT ANY
+WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
+PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+FLARE.  If not, see http://www.gnu.org/licenses/
+*/
+
 /**
  * class CampaignManager
  *
  * Contains data for story mode
- *
- * @author Clint Bellanger
- * @license GPL
  */
 
 #ifndef CAMPAIGN_MANAGER_H
@@ -14,13 +28,11 @@
 #include <sstream>
 #include "UtilsParsing.h"
 #include "MenuItemStorage.h"
-#include "ItemDatabase.h"
+#include "ItemManager.h"
 
 const int MAX_STATUS = 1024;
 
 class CampaignManager {
-private:
-
 public:
 	CampaignManager();
 	~CampaignManager();
@@ -44,7 +56,7 @@ public:
 	ItemStack drop_stack;
 	
 	// pointers to various info that can be changed
-	ItemDatabase *items;
+	ItemManager *items;
 	MenuItemStorage *carried_items;
 	int *currency;
 	int *xp;
