@@ -1,9 +1,22 @@
+/*
+Copyright 2011 Clint Bellanger
+
+This file is part of FLARE.
+
+FLARE is free software: you can redistribute it and/or modify it under the terms
+of the GNU General Public License as published by the Free Software Foundation,
+either version 3 of the License, or (at your option) any later version.
+
+FLARE is distributed in the hope that it will be useful, but WITHOUT ANY
+WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
+PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+FLARE.  If not, see http://www.gnu.org/licenses/
+*/
+
 /*
  * class Enemy
- *
- * @author Clint Bellanger
- * @license GPL
- *
  */
 
 #ifndef ENEMY_H
@@ -12,9 +25,9 @@
 
 #include <math.h>
 #include <string>
-#include "SDL.h"
-#include "SDL_image.h"
-#include "SDL_mixer.h"
+#include <SDL.h>
+#include <SDL_image.h>
+#include <SDL_mixer.h>
 
 #include "Entity.h"
 #include "Utils.h"
@@ -43,7 +56,7 @@ const int ENEMY_CRITDEAD = 10;
 class Enemy : public Entity {
 protected:
 	PowerManager *powers;
-	
+
 public:
 	Enemy(PowerManager *_powers, MapIso *_map);
 	~Enemy();
