@@ -1,34 +1,47 @@
+/*
+Copyright 2011 Clint Bellanger
+
+This file is part of FLARE.
+
+FLARE is free software: you can redistribute it and/or modify it under the terms
+of the GNU General Public License as published by the Free Software Foundation,
+either version 3 of the License, or (at your option) any later version.
+
+FLARE is distributed in the hope that it will be useful, but WITHOUT ANY
+WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
+PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+FLARE.  If not, see http://www.gnu.org/licenses/
+*/
+
 /**
  * MenuExperience
  *
  * Handles the display of the Experience bar on the HUD
- *
- * @author Clint Bellanger
- * @license GPL
  */
 
 #ifndef MENU_EXPERIENCE_H
 #define MENU_EXPERIENCE_H
 
-#include "SDL.h"
-#include "SDL_image.h"
-#include "SDL_mixer.h"
 #include "StatBlock.h"
 #include "Utils.h"
 #include "FontEngine.h"
+
+#include <SDL.h>
+#include <SDL_image.h>
+
 #include <string>
-#include <sstream>
+
 
 using namespace std;
 
 class MenuExperience {
 private:
-	SDL_Surface *screen;
-	FontEngine *font;
 	SDL_Surface *background;
 	SDL_Surface *bar;
 public:
-	MenuExperience(SDL_Surface *_screen, FontEngine *_font);
+	MenuExperience();
 	~MenuExperience();
 	void loadGraphics();
 	void render(StatBlock *stats, Point mouse);
@@ -46,7 +59,6 @@ public:
 	Point bar_size;
 	Point text_offset;
 	int text_justify;
-	string text_label;
 	
 };
 
